@@ -7,9 +7,9 @@ import lammps_step
 
 class EnergyStep(object):
     my_description = {
-        'description': 'A single-point energy calculation',
-        'group': 'Calculations',
-        'name': 'Energy'
+        "description": "A single-point energy calculation",
+        "group": "Calculations",
+        "name": "Energy",
     }
 
     def __init__(self, flowchart=None, gui=None):
@@ -20,8 +20,7 @@ class EnergyStep(object):
         pass
 
     def description(self):
-        """Return a description of what this extension does
-        """
+        """Return a description of what this extension does"""
         return EnergyStep.my_description
 
     def create_node(self, flowchart=None, **kwargs):
