@@ -2,10 +2,12 @@
 
 """NVT (canonical) dynamics in LAMMPS"""
 
+import json
+
 import lammps_step
 import logging
 import seamm
-from seamm_util import units_class
+from seamm_util import units_class, Q_
 import seamm_util.printing as printing
 from seamm_util.printing import FormattedText as __
 import random
@@ -43,25 +45,35 @@ thermostat_metadata = {
 
 
 class NVT(lammps_step.NVE):
-    def __init__(self, flowchart=None, title="NVT dynamics", extension=None):
+    def __init__(
+        self,
+        flowchart=None,
+        title="NVT dynamics",
+        extension=None,
+        logger=logger,
+    ):
         """Initialize the node"""
 
         logger.debug("Creating NVT {}".format(self))
 
-        super().__init__(flowchart=flowchart, title=title, extension=extension)
+        super().__init__(
+            flowchart=flowchart,
+            title=title,
+            extension=extension,
+            logger=logger,
+        )
 
-        logger.debug("NVT after super init, {}".format(self))
+        self.logger.debug("NVT after super init, {}".format(self))
 
         self.description = "NVT dynamics step in LAMMPS"
 
-        logger.debug("NVT.init() creating NVT_Parameters object")
+        self.logger.debug("NVT.init() creating NVT_Parameters object")
 
         self._calculation = "nvt"
-        self._model = None
         self._metadata = lammps_step.metadata
         self.parameters = lammps_step.NVT_Parameters()
 
-        logger.debug("NVT.init() completed")
+        self.logger.debug("NVT.init() completed")
 
     def description_text(self, P=None):
         """Return a short description of this step.
@@ -78,6 +90,8 @@ class NVT(lammps_step.NVE):
         if not P:
             P = self.parameters.values_to_dict()
 
+        model = self.model
+
         # What will we do?
 
         if P["T0"] == P["T1"]:
@@ -87,8 +101,13 @@ class NVT(lammps_step.NVE):
                 "{time} of canonical (NVT) dynamics starting "
                 " at {T0}, going to {T1}, "
             )
+        text += "using a timestep of {timestep} "
+        if model is None:
+            text += ". The temperature will be controlled "
+        else:
+            text += f"using the {model} forcefield. The temperature will be controlled "
         if P["thermostat"] == "Nose-Hoover":
-            text += "using a Nose-Hoover thermostat."
+            text += "using  a Nose-Hoover thermostat."
             if P["Tchain"] != "3":
                 if P["Tloop"] != "1":
                     text += (
@@ -167,6 +186,7 @@ class NVT(lammps_step.NVE):
         P = self.parameters.current_values_to_dict(
             context=seamm.flowchart_variables._data
         )
+        _, configuration = self.get_system_configuration()
 
         # Fix variables with special cases
         timestep, P["timestep"] = self.timestep(P["timestep"])
@@ -182,7 +202,7 @@ class NVT(lammps_step.NVE):
                 PP[key] = "{:~P}".format(PP[key])
 
         self.description.append(
-            __(self.description_text(PP), **PP, indent=3 * " ").__str__()
+            __(self.description_text(PP), **PP, indent=4 * " ").__str__()
         )
 
         time = lammps_step.to_lammps_units(P["time"], quantity="time")
@@ -198,16 +218,25 @@ class NVT(lammps_step.NVE):
         )
         properties = "v_time v_temp v_press v_etotal v_ke v_pe v_epair"
         title2 = "tstep t T P Etot Eke Epe Epair"
+        if configuration.periodicity == 3:
+            properties += " v_sxx v_syy v_szz v_sxy v_sxz v_syz"
+            title2 += " Sxx Syy Szz Sxy Sxz Syz"
+        if self.parent.have_dreiding_hbonds:
+            thermo_properties += " v_N_hbond v_E_hbond"
+            properties += " v_N_hbond v_E_hbond"
+            title2 += " N_hbond E_hbond"
 
         lines = []
         lines.append("")
-        lines.append("#     NVT dynamics")
+        lines.append(f"# {self.header}")
         lines.append("")
         lines.append("reset_timestep      0")
         lines.append("timestep            {}".format(timestep))
         lines.append("thermo_style        custom {}".format(thermo_properties))
         lines.append("thermo              {}".format(int(nsteps / 100)))
 
+        ncomputes = 0
+        ndumps = 0
         nfixes = 0
         if P["thermostat"] == "Nose-Hoover":
             Tchain = P["Tchain"]
@@ -253,7 +282,7 @@ class NVT(lammps_step.NVE):
         elif P["thermostat"] == "velocity rescaling":
             frequency = lammps_step.to_lammps_units(P["frequency"], quantity="time")
 
-            nevery = round(nsteps / (frequency / timestep))
+            nevery = max(1, round(nsteps / (frequency / timestep)))
             window = lammps_step.to_lammps_units(P["window"], quantity="temperature")
             fraction = P["fraction"]
             nfixes += 1
@@ -288,6 +317,36 @@ class NVT(lammps_step.NVE):
                 level=1,
                 note="Citation for thermostat.",
             )
+        # For the heat flux, if requested, we need extra input
+        if P["heat flux"] != "never":
+            # Unit conversion factor
+            if lammps_step.get_lammps_unit_system() == "metal":
+                factor = Q_("eV/Å^2/ps")
+            else:
+                factor = (
+                    Q_("kcal/Å^2/fs/mol") / Q_("kcal/mol") * Q_("kcal/mol").to("kJ")
+                )
+            factor = factor.m_as("W/m^2")
+            lines.append(
+                f"""
+compute             KE all ke/atom
+compute             PE all pe/atom
+
+#          centroid doesn't work with kspace, so split into pair and non-pair parts
+
+compute             S_p all stress/atom NULL pair kspace
+compute             S_b all centroid/stress/atom NULL bond angle dihedral improper
+compute             flux_p all heat/flux KE PE S_p
+compute             flux_b all heat/flux KE PE S_b
+
+#          Conversion from kcal/Å^2/fs/mol to W/m^2")
+
+variable            factor equal {factor}
+variable            Jx equal v_factor*(c_flux_p[1]+c_flux_b[1])/vol
+variable            Jy equal v_factor*(c_flux_p[2]+c_flux_b[2])/vol
+variable            Jz equal v_factor*(c_flux_p[3]+c_flux_b[3])/vol
+"""
+            )
 
         # summary output written 10 times during run so we can see progress
         nevery = 10
@@ -295,19 +354,12 @@ class NVT(lammps_step.NVE):
         nrepeat = int(nfreq / nevery)
         nfreq = nevery * nrepeat
         nfixes += 1
+        filename = f"@{self._id[-1]}+nvt_summary.trj"
         lines.append(
-            (
-                "fix                 {} all ave/time {} {} {} {} off 2 "
-                "title2 '{}' file summary_nvt_{}.txt"
-            ).format(
-                nfixes,
-                nevery,
-                nrepeat,
-                nfreq,
-                properties,
-                title2,
-                "_".join(str(e) for e in self._id),
-            )
+            f"fix                 {nfixes} all ave/time {nevery} 1 {nfreq} &\n"
+            f"                       {properties} &\n"
+            "                       off 2 &\n"
+            f"                       file {filename}"
         )
         # instantaneous output written for averaging
         if P["sampling"] == "none":
@@ -320,33 +372,48 @@ class NVT(lammps_step.NVE):
             )
         else:
             sampling = lammps_step.to_lammps_units(P["sampling"], quantity="time")
-            nevery = round(sampling / timestep)
+            nevery = max(1, round(sampling / timestep))
             nrepeat = 1
             nfreq = nevery * nrepeat
             nfixes += 1
 
+            dt = (nevery * P["timestep"]).to_compact()
             if T0 == T1:
-                title1 = (
-                    "!MolSSI trajectory 1.0 LAMMPS, NVT {} steps of {} fs, " "T={} K"
-                ).format(int(nsteps / nevery), timestep * nevery, T0)
-            else:
-                title1 = (
-                    "!MolSSI trajectory 1.0 LAMMPS, NVT {} steps of {} fs, " "T={}-{} K"
-                ).format(nsteps, timestep, T0, T1)
-            lines.append(
-                (
-                    "fix                 {} all ave/time {} {} {} {} off 2"
-                    " title1 '{}' title2 '{}' file trajectory_nvt_{}.seamm_trj"
-                ).format(
-                    nfixes,
-                    nevery,
-                    nrepeat,
-                    nfreq,
-                    properties,
-                    title1,
-                    title2,
-                    "_".join(str(e) for e in self._id),
+                text = json.dumps(
+                    {
+                        "code": "LAMMPS",
+                        "type": "NVT",
+                        "dt": dt.magnitude,
+                        "tunits": str(dt.u),
+                        "nsteps": nsteps // nevery,
+                        "T": T0,
+                        "Tunits": "K",
+                    },
+                    separators=(",", ":"),
                 )
+            else:
+                text = json.dumps(
+                    {
+                        "code": "LAMMPS",
+                        "type": "NVT",
+                        "dt": dt.magnitude,
+                        "units": str(dt.u),
+                        "nsteps": nsteps // nevery,
+                        "T0": T0,
+                        "T1": T1,
+                        "Tunits": "K",
+                    },
+                    separators=(",", ":"),
+                )
+            title1 = "!MolSSI trajectory 2.0 " + text
+            filename = f"@{self._id[-1]}+nvt_state.trj"
+            lines.append(
+                f"fix                 {nfixes} all ave/time {nevery} 1 {nfreq} &\n"
+                f"                       {properties} &\n"
+                "                       off 2 &\n"
+                f"                       title1 '{title1}' &\n"
+                f"                       title2 '{title2}' &\n"
+                f"                       file {filename}"
             )
             self.description.append(
                 __(
@@ -360,6 +427,12 @@ class NVT(lammps_step.NVE):
                 )
             )
 
+        # Handle trajectories
+        tmp, ncomputes, ndumps, nfixes = self.trajectory_input(
+            P, timestep, nsteps, ncomputes, ndumps, nfixes
+        )
+        lines.extend(tmp)
+
         if extras is not None and "shake" in extras:
             nfixes += 1
             lines.append(extras["shake"].format(nfixes))
@@ -367,9 +440,29 @@ class NVT(lammps_step.NVE):
         lines.append("")
         lines.append("run                 {}".format(nsteps))
         lines.append("")
-
-        for fix in range(1, nfixes + 1):
-            lines.append("unfix               {}".format(fix))
+        filename = f"@{self._id[-1]}+nvt.dump"
+        if configuration.periodicity == 0:
+            lines.append(
+                f"write_dump         all custom  {filename} id xu yu zu vx vy vz"
+                " modify flush yes sort id"
+            )
+        else:
+            lines.append(
+                f"write_dump         all custom  {filename} id xsu ysu zsu vx vy vz"
+                " modify flush yes sort id"
+            )
         lines.append("")
 
-        return lines
+        for i in range(1, ncomputes + 1):
+            lines.append(f"uncompute           {i}")
+        for i in range(1, ndumps + 1):
+            lines.append(f"undump              {i}")
+        for i in range(1, nfixes + 1):
+            lines.append(f"unfix               {i}")
+        lines.append("")
+
+        return {
+            "script": lines,
+            "postscript": None,
+            "use python": False,
+        }
